/////////////////////////////////////////////////////////////////////////////

var video_mode = 8;
var video_depth = 32;

var motor_right;
var motor_left;
var joints[4];
var index = 0;

/////////////////////////////////////////////////////////////////////////////

string carph_wmb = <carph.wmb>;
string car_mdl = <car.mdl>;
string tyreleft_mdl = <tyrel.mdl>;
string tyreright_mdl = <tyrer.mdl>;

/////////////////////////////////////////////////////////////////////////////

function players_car();
function cars_tyre();
function flip_it();

/////////////////////////////////////////////////////////////////////////////

function main()
{
	time_smooth = 0; // don't smooth the "time" variable (prevents jerks)
	fps_max = 140; // set fps_max to 70 x 2 = 140 (the physics engine is locked at 70 fps)
	fps_lock = on; // lock "time" to fps_max
	level_load (carph_wmb);
	wait (3);
	ph_setgravity (vector(0, 0, -386)); // we're going to use a gravity vector that's similar to Earth's gravity
	ent_create (car_mdl, vector(0, 0, 100), players_car); // create the car 100 quants above the origin of the z axis
	while(player == null) {wait(1);} // wait until the player is created
	while (1)
	{
		motor_left = 0; // reset the motors
		motor_right = 0;
		if(key_cuu == on) // cursor up was pressed?
		{
			motor_left = -20; // then allow both motors to move
			motor_right = -20;
		}
		if(key_cud == on) // cursor down was pressed?
		{
			motor_left = 20; // then allow both motors to move
			motor_right = 20;
		}
		if(key_cul == on) // cursor left was pressed?
		{
			motor_left = 0; // allow movement only for the wheels on the right side of the car
		}
		if(key_cur == on) // cursor right was pressed?
		{	
			motor_right = 0; // allow movement only for the wheels on the left side of the car
		}
		phcon_setmotor (joints[0], vector(motor_right, 10000, 0), nullvector, nullvector); // turn the joints into motorized joints
		phcon_setmotor (joints[2], vector(motor_right, 10000, 0), nullvector, nullvector); // limit the force to 10,000
		phcon_setmotor (joints[1], vector(motor_left, 10000, 0), nullvector, nullvector);
		phcon_setmotor (joints[3], vector(motor_left, 10000, 0), nullvector, nullvector);

		camera.x = player.x - 300 * cos(player.pan); // keep the camera 300 quants behind the player
		camera.y = player.y - 300 * sin(player.pan);  
		camera.z = player.z + 1000; // and 1000 quants above
		camera.pan = player.pan; // the camera uses the same pan angle with the player
		camera.tilt = -60; // and it points its nose downwards, looking at the player

		wait (1);
	}
}

function players_car() // this function gets attached to the body of the car
{
	player = my; // I'm the player
	my.scale_x = 1.5; // change this value to modify the scale of the car on the x y z axis
	my.scale_y = my.scale_x;
	my.scale_z = my.scale_x;
	my.shadow = on;
	phent_settype (my, ph_rigid, ph_box); // I've got a rigid body and I behave like a box as far as collision detection is concerned
	phent_setmass (my, 100, ph_box); // I weigh 100 kilograms (~220 pounds)
	phent_setgroup (my, 2); // set the id of the car
	phent_setfriction (my, 20); // set the friction coefficient
	phent_setdamping (my, 20, 30); // set the velocity damping (linear = 20, angular = 30)
	phent_setelasticity (my, 30, 10); // set the bounciness factor
	ent_create(tyreleft_mdl, vector(my.x - 65, my.y - 43, my.z - 35), cars_tyre); // create the rear left wheel
	ent_create(tyreright_mdl, vector(my.x - 65, my.y + 43, my.z - 35), cars_tyre); // create the rear right wheel
	ent_create(tyreleft_mdl, vector(my.x + 48, my.y - 43, my.z - 35), cars_tyre); // create the front left wheel
	ent_create(tyreright_mdl, vector(my.x + 48, my.y + 43, my.z - 35), cars_tyre); // create the front right wheel
}

function cars_tyre()
{
	my.shadow = on;
	my.scale_x = 2; // set the scale of the wheels, play with this value
	my.scale_y = my.scale_x;
	my.scale_z = my.scale_x;
	phent_settype (my, ph_rigid, ph_sphere); // the wheels have a rigid body and behave like spheres
	phent_setmass (my, 10, ph_sphere); // they've got 10 kilograms each (~22 pounds)
	phent_setgroup (my, 2); // set the id for the wheels
	phent_setfriction (my, 60); // set the friction coefficient
	phent_setdamping (my, 40, 5); // and the damping factor
	phent_setelasticity (my, 30, 10); // set the bounciness
	joints[index] = phcon_add (ph_hinge, player, my); // bind the wheel to the player (the car), returns the identifier of the constraint
	// set the anchor point for the hinge to the origin of the wheel model and the hinge axis to y (0, 1, 0)
	phcon_setparams1 (joints[index], my.x, vector(0, 1, 0), nullvector); 
	// allowed angles for the joint (-360, 360, 0) sets no limits
	phcon_setparams2 (joints[index], vector(-360, 360, 0), nullvector, nullvector);
	index += 1; // increase the index for the following wheel
}

function flip_it() // flip the car if it rolls over
{
	phent_addtorquelocal (player, vector((random(-200000) + 100000), 0, 0));
}

action barrel
{
	my.shadow = on;
	phent_settype (my, ph_rigid, ph_box); // the barrels have a rigid body and behave like boxes
	phent_setmass (my, 20, ph_box); // they've got 20 kilograms (~44 pounds)
	phent_setfriction (my, 50); // set the friction coefficient
	phent_setdamping (my, 50, 50); // set the damping factor
	phent_setelasticity (my, 30, 30); // set the bounciness
}

on_mouse_right = flip_it; // press the right mouse button to flip the car
